import { DoubleArray } from 'cheminfo-types';

import { applyWindow } from './applyWindow';
import { compose, ShapeOptions } from './compose';

export interface DataReIm {
  /** Array of re values */
  re: DoubleArray;
  /** Array of im values */
  im: DoubleArray;
}

export interface ApodizationFilterOptions {
  /**
   * number of points at the end of the shape to apply the window function inverted
   * @default 0
   */
  pointsToShift?: number;
  compose?: ShapeOptions;
  /**
   * apply the window data to input data
   * @default true
   */
  apply?: boolean;
}
export function apodization(
  data: DataReIm,
  options: ApodizationFilterOptions = {},
) {
  const { compose: composeOptions, pointsToShift, apply = true } = options;

  if (!composeOptions) {
    return { ...data, windowData: new Float64Array(data.re.length).fill(1) };
  }

  const windowData = compose(composeOptions);

  const applyWindowOptions = {
    windowData,
    pointsToShift,
  };

  const re = apply ? applyWindow(data.re, applyWindowOptions) : data.re;
  const im = apply ? applyWindow(data.im, applyWindowOptions) : data.im;

  return { re, im, windowData };
}
