import { DoubleArray } from 'cheminfo-types';

export interface ApplyWindowOptions {
  /**
   * data of a window function it should be scaled to max = 1 to ensure compatibility with new apodizations.
   */
  windowData: DoubleArray;
  /**
   * Start Location for Applying Apodize Window
   */
  start?: number;
  /**
   * length of the window function
   */
  length?: number;
  /**
   * number of points at the end of the shape to apply the window function inverted
   * @default 0
   */
  pointsToShift?: number;
  /**
   * output array
   */
  output?: DoubleArray;
}

/**
 * pure function that applies a window function to the input data.
 */
export function applyWindow(data: DoubleArray, options: ApplyWindowOptions) {
  const dataLength = data.length;

  const {
    windowData,
    start = 0,
    length = dataLength,
    pointsToShift = 0,
    output = new Float64Array(data),
  } = options;

  const firstEndPoint = Math.min(start + length, dataLength - pointsToShift);
  for (let i = start, j = 0; i < firstEndPoint; i++) {
    output[i] *= windowData[j++];
  }
  for (let i = dataLength - 1, j = 0; i > dataLength - pointsToShift - 1; i--) {
    output[i] *= windowData[j++];
  }

  return output;
}
