import {
  Predictions1Dassignments,
  StoreAssignments,
} from './oneDimensionalAssignment/buildAssignments';

export interface SolutionAssignment {
  assignment: string[];
  score: number;
}

interface AddSolutionProps {
  score: number;
  maxSolutions: number;
  partial: Array<string | null>;
  predictions: Predictions1Dassignments;
}

export function addSolution(store: StoreAssignments, props: AddSolutionProps) {
  const { score, maxSolutions, partial, predictions } = props;
  store.nSolutions++;
  const solution: SolutionAssignment = {
    assignment: JSON.parse(JSON.stringify(partial)),
    score: score / doubleAssignmentPenalty(partial, predictions),
  };

  if (store.nSolutions >= maxSolutions) {
    if (solution.score > store.solutions.last().score) {
      store.solutions.pollLast();
      store.solutions.add(solution);
    }
  } else {
    store.solutions.add(solution);
    store.nSolutions++;
  }
}

function doubleAssignmentPenalty(
  partial: Array<string | null>,
  predictions: Predictions1Dassignments,
) {
  const nbSources = Object.keys(predictions).length;
  const assignments = new Set(partial);
  const nbDoubleAssignment = nbSources - assignments.size;
  return nbDoubleAssignment > 0 ? 2 * nbDoubleAssignment : 1;
}
