import { v4 as generateID } from '@lukeed/uuid';

import type { NMRRange, NMRZone } from '../../..';
import type { SpectraData } from '../../getAssignments';
import type {
  SpectraDataWithIds,
  NMRRangeWithIds,
  NMRZoneWithIds,
} from '../../nmrAssigment';

import { isSpectraData1D } from './isSpectraData1D';

export function checkIDs(input: SpectraData[] = []): SpectraDataWithIds[] {
  const inputClone = JSON.parse(JSON.stringify(input)) as SpectraData[];
  const spectra: SpectraDataWithIds[] = [];
  for (const spectraData of inputClone) {
    if (!('info' in spectraData)) {
      throw new Error(
        'each spectrum should contain info properties with nucleus information',
      );
    }

    const { info, id } = spectraData;
    if (isSpectraData1D(spectraData)) {
      const ranges = addIDs(spectraData.ranges) as NMRRangeWithIds[];
      spectra.push({ id, info, ranges });
    } else {
      const data = addIDs(spectraData.zones) as NMRZoneWithIds[];
      spectra.push({ id, info, zones: data });
    }
  }
  return spectra;
}

export function hasIDs(
  data: NMRRange[] | NMRZone[],
): asserts data is NMRRangeWithIds[] | NMRZoneWithIds[] {
  for (const element of data) {
    if (!element.id) throw new Error('A range/zone has not an ID');
    for (const signal of element.signals || []) {
      if (!signal.id) throw new Error('A signal has not an ID');
    }
  }
}

export function addIDs<T extends NMRRange[] | NMRZone[]>(data: T) {
  for (const element of data) {
    if (!element.id) element.id = generateID();
    for (const signal of element.signals || []) {
      if (!signal.id) signal.id = generateID();
    }
  }
  hasIDs(data);
  return data;
}
