import {
  NMRRangeWithIds,
  NMRZoneWithIds,
  SpectraData1DWithIds,
  SpectraData2DWithIds,
  SpectraDataWithIds,
} from '../../nmrAssigment';

import { isSpectraData1D } from './isSpectraData1D';

interface SpectraData1DFormatted extends Omit<SpectraData1DWithIds, 'ranges'> {
  ranges: { values: NMRRangeWithIds[] };
}
interface SpectraData2DFormatted extends Omit<SpectraData2DWithIds, 'zones'> {
  zones: { values: NMRZoneWithIds[] };
}

export type SpectraDataFormatted =
  | SpectraData1DFormatted
  | SpectraData2DFormatted;

export function formatData(
  input: SpectraDataWithIds[] = [],
): SpectraDataFormatted[] {
  const inputClone = JSON.parse(JSON.stringify(input)) as SpectraDataWithIds[];
  const spectra: SpectraDataFormatted[] = [];
  for (const spectraData of inputClone) {
    const { id, info } = spectraData;
    if (isSpectraData1D(spectraData)) {
      const ranges = rescaleIntegration({
        id,
        info,
        ranges: spectraData.ranges,
      });
      spectra.push({ id, info, ranges: { values: ranges } });
    } else {
      spectra.push({ id, info, zones: { values: spectraData.zones } });
    }
  }
  return spectra;
}

function rescaleIntegration(spectrum: SpectraData1DWithIds): NMRRangeWithIds[] {
  if (spectrum.info.nucleus !== '1H') return spectrum.ranges;

  const totalIntegration = spectrum.ranges.reduce(
    (total, range) => (range.integration ? total + range.integration : total),
    0,
  );

  for (const range of spectrum.ranges) {
    if (range.integration) {
      range.integration *= 100 / totalIntegration;
      if (range.signals) {
        const totalSignalIntegration = range.signals.reduce(
          (total, signal) =>
            signal.integration ? total + signal.integration : total,
          0,
        );
        for (const signal of range.signals) {
          if (signal.integration) {
            signal.integration *= range.integration / totalSignalIntegration;
          } else {
            signal.integration = range.integration;
          }
        }
      }
    }
  }

  return spectrum.ranges;
}
