import { MultiplicityPatterns } from '../MultiplicityPatterns';
import { couplingPatterns } from '../couplingPatterns';
import { couplingValues } from '../couplingValues';

describe('check multiplicities', () => {
  it('check a singlet', () => {
    const index = couplingValues.s;
    expect(index).toBe(0);
    expect(couplingPatterns[index]).toBe('s');
  });
  it('check a doublet', () => {
    const index = couplingValues.d;
    expect(index).toBe(1);
    expect(couplingPatterns[index]).toBe('d');
  });
  it('check a triplet', () => {
    const index = couplingValues.t;
    expect(index).toBe(2);
    expect(couplingPatterns[index]).toBe('t');
  });
  it('check a quadruplet', () => {
    const index = couplingValues.q;
    expect(index).toBe(3);
    expect(couplingPatterns[index]).toBe('q');
  });
  it('check a quintet', () => {
    const multiplet = MultiplicityPatterns.find((m) => m.label === 'quintet');

    expect(multiplet).toBeDefined();
    const { names = [], value = '', multiplicity = 0 } = multiplet || {};
    expect(names.length).toBeGreaterThan(1);
    expect(value).toBe('i');
    expect(multiplicity).toBeGreaterThan(1);
    for (const name of names) {
      const index = couplingValues[name];
      expect(index).toBe(4);
      expect(couplingPatterns[index]).toBe('i');
    }
  });
  it('check a hexuplet', () => {
    const multiplet = MultiplicityPatterns.find((m) => m.label === 'sextet');

    expect(multiplet).toBeDefined();
    const { names = [], value = '', multiplicity = 0 } = multiplet || {};
    expect(names.length).toBeGreaterThan(1);
    expect(value).toBe('h');
    expect(multiplicity).toBeGreaterThan(1);
    for (const name of names) {
      const index = couplingValues[name];
      expect(index).toBe(5);
      expect(couplingPatterns[index]).toBe('h');
    }
  });
  it('check a septet', () => {
    const multiplet = MultiplicityPatterns.find((m) => m.label === 'septet');

    expect(multiplet).toBeDefined();
    const { names = [], value = '', multiplicity = 0 } = multiplet || {};
    expect(names.length).toBeGreaterThan(1);
    expect(value).toBe('p');
    expect(multiplicity).toBeGreaterThan(1);
    for (const name of names) {
      const index = couplingValues[name];
      expect(index).toBe(6);
      expect(couplingPatterns[index]).toBe('p');
    }
  });
  it('check a octet', () => {
    const multiplet = MultiplicityPatterns.find((m) => m.label === 'octet');
    expect(multiplet).toBeDefined();
    const { names = [], value = '', multiplicity = 0 } = multiplet || {};
    expect(names.length).toBeGreaterThan(0);
    expect(value).toBe('o');
    expect(multiplicity).toBeGreaterThan(1);
    for (const name of names) {
      const index = couplingValues[name];
      expect(index).toBe(7);
      expect(couplingPatterns[index]).toBe('o');
    }
  });
  it('check a nonet', () => {
    const multiplet = MultiplicityPatterns.find((m) => m.label === 'nonet');
    expect(multiplet).toBeDefined();
    const { names = [], value = '', multiplicity = 0 } = multiplet || {};
    expect(names.length).toBeGreaterThan(0);
    expect(value).toBe('n');
    expect(multiplicity).toBeGreaterThan(1);
    for (const name of names) {
      const index = couplingValues[name];
      expect(index).toBe(8);
      expect(couplingPatterns[index]).toBe('n');
    }
  });
});
