import { Filter } from '../../../../types/Filter';
import { Filters } from '../../../Filters';
import { applyFilter } from '../../../FiltersManager';
import { Entry1D } from '../../Entry1D';

describe('lineBroadening', () => {
  const entry: Entry1D = {
    data: {
      re: new Float64Array([1, 2, 3, 4]),
      im: new Float64Array([0, 0, 0, 0]),
      x: new Float64Array([10, 20, 30, 40]),
    },
    info: {
      name: 'testApodization',
      nucleus: '1H',
      spectralWidth: 10,
      originFrequency: 80,
      dimension: 1,
      isFt: false,
      isFid: true,
      isComplex: true,
    },
    peaks: { values: [], options: {} },
    ranges: {
      values: [],
      options: {
        sum: undefined,
        isSumConstant: true,
        sumAuto: true,
      },
    },
    integrals: {
      values: [],
      options: {
        sum: undefined,
        isSumConstant: true,
        sumAuto: true,
      },
    },
    filters: [] as Filter[],
  };
  it('simple x, re, im to 1 Hz exp.', () => {
    const spectrum: Entry1D = { ...entry };
    applyFilter(spectrum, [
      {
        name: Filters.apodization.id,
        value: {
          lineBroadening: 0.1,
          gaussBroadening: 0,
          lineBroadeningCenter: 0,
        },
      },
    ]);

    expect(spectrum.data).toStrictEqual({
      re: new Float64Array([
        1, 0.08642783652754452, 0.0056023281951239675, 0.0003227980702812185,
      ]),
      im: new Float64Array([0, 0, 0, 0]),
      x: new Float64Array([10, 20, 30, 40]),
    });
    const filter = spectrum.filters[0];
    expect(filter.name).toBe(Filters.apodization.id);
    expect(filter.value).toEqual(
      expect.objectContaining({
        lineBroadening: 0.1,
        gaussBroadening: 0,
        lineBroadeningCenter: 0,
      }),
    );
  });
  it('without apodization options.', () => {
    const spectrum: Entry1D = { ...entry };
    applyFilter(spectrum, [
      {
        name: Filters.apodization.id,
        value: {},
      },
    ]);
    const filter = spectrum.filters[0];
    expect(filter.name).toBe(Filters.apodization.id);

    expect(filter.value).toEqual(
      expect.objectContaining({
        lineBroadening: 1,
        gaussBroadening: 0,
        lineBroadeningCenter: 0,
      }),
    );
  });
});
