import { NmrData1D } from 'cheminfo-types';

import { apodization } from '../../../../apodization/apodization';
import { Info1D } from '../../../../types/Info/Info1D';
import { Entry1D } from '../../Entry1D';

import { ApodizationOptions } from './apodization';

type Entry1DApodizationFilter = Partial<Omit<Entry1D, 'info'>> & {
  info: Partial<Info1D>;
  data: Required<NmrData1D>;
};

export function apodizationFilter(
  datum1D: Entry1DApodizationFilter,
  options: ApodizationOptions,
) {
  const { lineBroadening, gaussBroadening, lineBroadeningCenter } = options;

  const grpdly = datum1D.info?.digitalFilter || 0;
  const pointsToShift = grpdly > 0 ? Math.floor(grpdly) : 0;

  const re = datum1D.data.re;
  const im = datum1D.data.im;
  const t = datum1D.data.x;

  const length = re.length;
  const dw = (t[length - 1] - t[0]) / (length - 1); //REPLACE CONSTANT with calculated value... : for this we need AQ or DW to set it right...

  return apodization(
    { re, im },
    {
      pointsToShift,
      compose: {
        length,
        shapes: [
          {
            start: 0,
            shape: {
              kind: 'lorentzToGauss',
              options: {
                length,
                dw,
                exponentialHz:
                  gaussBroadening > 0 ? lineBroadening : -lineBroadening,
                gaussianHz: gaussBroadening,
                center: lineBroadeningCenter,
              },
            },
          },
        ],
      },
    },
  );
}
