import { FilterDomainUpdateRules } from '../../FiltersManager';
import { Entry1D } from '../Entry1D';

/**
 *
 * @param {Object} datum1d
 * @param {number} [shiftValue=0]
 */

export const id = 'shiftX';
export const name = 'Shift X';

export const DOMAIN_UPDATE_RULES: Readonly<FilterDomainUpdateRules> = {
  updateXDomain: true,
  updateYDomain: false,
};

interface ShiftXOptions {
  shift?: number;
}
export function apply(spectrum: Entry1D, options: ShiftXOptions = {}) {
  const { shift = 0 } = options;
  spectrum.data.x = spectrum.data.x.map((val) => val + shift);
}

export function isApplicable() {
  return true;
}

export function reduce(previous: ShiftXOptions, next: ShiftXOptions) {
  const { shift: previousValue = 0 } = previous;
  const { shift: nextShift = 0 } = next;
  return {
    once: true,
    reduce: { shift: previousValue + nextShift },
  };
}
