import { v4 } from '@lukeed/uuid';

import { Peak1D } from '../../../types/Peaks/Peak1D';
import { MapOptions, ShiftTarget } from '../../../types/common/MapOptions';
import { Entry1D } from '../Entry1D';

import { getShiftX } from './getShiftX';
import { getSpectrumErrorValue } from './getSpectrumErrorValue';

function isExists(peak: Peak1D, spectrum: Entry1D, error: number) {
  // check if the Peak is already exists
  for (const { x } of spectrum.peaks?.values || []) {
    if (Math.abs(peak.x - x) < error) {
      return true;
    }
  }
  return false;
}

function getPeakDelta(peak: Peak1D, shiftTarget: ShiftTarget, shift: number) {
  const { originalX = 0, x } = peak;
  if (shiftTarget === 'origin') {
    return {
      originalX: x - shift,
      x,
    };
  } else {
    return {
      originalX,
      x: originalX + shift,
    };
  }
}

export function mapPeaks(
  peaks: Peak1D[],
  spectrum: Entry1D,
  options: MapOptions = {},
) {
  const { checkIsExisting = true, shiftTarget = 'origin' } = options;
  const shiftX = getShiftX(spectrum);
  const error = getSpectrumErrorValue(spectrum);

  const newPeaks: Peak1D[] = [];

  for (const peak of peaks) {
    const peakDelta = getPeakDelta(peak, shiftTarget, shiftX);

    if (
      !checkIsExisting ||
      (checkIsExisting && !isExists(peak, spectrum, error))
    ) {
      newPeaks.push({
        ...peak,
        id: peak?.id || v4(),
        ...peakDelta,
      });
    }
  }

  return newPeaks;
}
