import { Range } from '../../../types/Ranges/Ranges';
import { getSum } from '../../../utilities/getSum';
import { Entry1D } from '../Entry1D';

import { checkRangeKind } from './checkRangeKind';
import { mapValues } from './mapValues';

export function updateRangesRelativeValues(
  spectrum: Entry1D,
  forceCalculateIntegral = false,
) {
  const { values, options } = spectrum.ranges;

  const currentSum = getSum(values, 'absolute', checkRangeKind);

  let factor = 0;
  if (options?.sum) {
    if (options.isSumConstant || forceCalculateIntegral) {
      factor = currentSum > 0 ? options.sum / currentSum : 0;
    } else if (values?.[0]) {
      const item = values[0];
      factor = (item.integration || options.sum) / item.absolute;
    }
  }

  spectrum.ranges.values = mapValues(
    values,
    'integration',
    factor,
    checkRangeKind,
  ) as Range[];

  if (!options.isSumConstant && !forceCalculateIntegral) {
    options.sum = getSum(values, 'integration', checkRangeKind);
  }
}
