import { NmrData2DContent } from 'cheminfo-types';

import { FilterDomainUpdateRules } from '../../FiltersManager';
import { Entry2D } from '../Entry2D';

export const id = 'shift2DX';
export const name = 'Shift 2D X';

export const DOMAIN_UPDATE_RULES: Readonly<FilterDomainUpdateRules> = {
  updateXDomain: true,
  updateYDomain: false,
};

interface Shift2DXOptions {
  shift: number;
}
export function apply(datum2D: Entry2D, options: Shift2DXOptions) {
  const { shift } = options;
  const data: Record<string, NmrData2DContent> = { ...datum2D.data };
  for (const key in data) {
    data[key].minX += shift;
    data[key].maxX += shift;
  }
}

export function isApplicable() {
  return true;
}

export function reduce(previous: Shift2DXOptions, next: Shift2DXOptions) {
  const { shift: previousValue = 0 } = previous;
  const { shift: nextShift } = next;

  return {
    once: true,
    reduce: { shift: previousValue + nextShift },
  };
}
