import { NmrData2DContent } from 'cheminfo-types';

import { FilterDomainUpdateRules } from '../../FiltersManager';
import { Entry2D } from '../Entry2D';

export const id = 'shift2DY';
export const name = 'Shift 2D Y';

export const DOMAIN_UPDATE_RULES: Readonly<FilterDomainUpdateRules> = {
  updateXDomain: false,
  updateYDomain: true,
};

interface Shift2DYOptions {
  shift: number;
}

export function apply(datum2D: Entry2D, options: Shift2DYOptions) {
  const { shift } = options;
  const data: Record<string, NmrData2DContent> = { ...datum2D.data };
  for (const key in data) {
    data[key].minY += shift;
    data[key].maxY += shift;
  }
}

export function isApplicable() {
  return true;
}

export function reduce(previous: Shift2DYOptions, next: Shift2DYOptions) {
  const { shift: previousValue = 0 } = previous;
  const { shift: nextShift } = next;
  return {
    once: true,
    reduce: { shift: previousValue + nextShift },
  };
}
