import { findMultiplet } from '../findMultiplet';

describe('find a multiplet from string', () => {
  it('broad singlet', () => {
    const multiplet = findMultiplet('br. s');
    expect(multiplet).toBeDefined();
    const { label, multiplicity, value } = multiplet || {};
    expect(label).toBe('singlet');
    expect(value).toBe('s');
    expect(multiplicity).toEqual(1);
  });
  it('triplet', () => {
    const multiplet = findMultiplet('triplet');
    expect(multiplet).toBeDefined();
    const { label, multiplicity, value } = multiplet || {};
    expect(label).toBe('triplet');
    expect(value).toBe('t');
    expect(multiplicity).toEqual(3);
  });
  it('triplet from t', () => {
    const multiplet = findMultiplet('t');
    expect(multiplet).toBeDefined();
    const { label, multiplicity, value } = multiplet || {};
    expect(label).toBe('triplet');
    expect(value).toBe('t');
    expect(multiplicity).toEqual(3);
  });
  it('quartet', () => {
    const multiplet = findMultiplet('quartet');
    expect(multiplet).toBeDefined();
    const { label, multiplicity, value } = multiplet || {};
    expect(label).toBe('quartet');
    expect(value).toBe('q');
    expect(multiplicity).toEqual(4);
  });
  it('quartet from q', () => {
    const multiplet = findMultiplet('q');
    expect(multiplet).toBeDefined();
    const { label, multiplicity, value } = multiplet || {};
    expect(label).toBe('quartet');
    expect(value).toBe('q');
    expect(multiplicity).toEqual(4);
  });
  it('quintet', () => {
    const multiplet = findMultiplet('quint');
    expect(multiplet).toBeDefined();
    const { label, multiplicity, value, acs } = multiplet || {};
    expect(label).toBe('quintet');
    expect(acs).toBe('quint');
    expect(value).toBe('i');
    expect(multiplicity).toEqual(5);
  });
  it('quintet from i', () => {
    const multiplet = findMultiplet('i');
    expect(multiplet).toBeDefined();
    const { label, multiplicity, value, acs } = multiplet || {};
    expect(label).toBe('quintet');
    expect(acs).toBe('quint');
    expect(value).toBe('i');
    expect(multiplicity).toEqual(5);
  });
  it('quintet from quintet', () => {
    const multiplet = findMultiplet('quintet');
    expect(multiplet).toBeDefined();
    const { label, multiplicity, value, acs } = multiplet || {};
    expect(label).toBe('quintet');
    expect(acs).toBe('quint');
    expect(value).toBe('i');
    expect(multiplicity).toEqual(5);
  });
  it('sept from sept.', () => {
    const multiplet = findMultiplet('sept.');
    expect(multiplet).toBeDefined();
    const { label, multiplicity, value, acs } = multiplet || {};
    expect(label).toBe('septet');
    expect(acs).toBe('sept');
    expect(value).toBe('p');
    expect(multiplicity).toEqual(7);
  });
  it('massive', () => {
    const multiplet = findMultiplet('massive');
    expect(multiplet).toBeDefined();
    const { label, multiplicity, value } = multiplet || {};
    expect(label).toBe('massive');
    expect(value).toBe('m');
    expect(multiplicity).toEqual(null);
  });
  it('massive from m', () => {
    const multiplet = findMultiplet('m');
    expect(multiplet).toBeDefined();
    const { label, multiplicity, value } = multiplet || {};
    expect(label).toBe('massive');
    expect(value).toBe('m');
    expect(multiplicity).toEqual(null);
  });
  it('should return empty', () => {
    expect(findMultiplet('world')).not.toBeDefined();
  });
});
