import {
  couplingACSPatterns,
  couplingPatterns,
} from '../constants/couplingPatterns';
import { couplingValues } from '../constants/couplingValues';

/**
 * join a list of pattern assuming that the coupling constant is the same. e.g ['d', 'd'] will returns 't'.
 * @param patterns - list of patterns name or abbreviations.
 * @returns
 */

export interface JoinPatternsOptions {
  /**
   * if true, the pattern returned will follow the ACS guide line.
   * @default false
   */
  acsFormat?: boolean;
}

export function joinPatterns(
  patterns: string[],
  options?: JoinPatternsOptions,
) {
  const sum = patterns.reduce((sum, pattern) => {
    if (isNaN(couplingValues[pattern])) {
      throw new Error(
        `pattern ${pattern} is not in ${Object.keys(couplingValues).join(' ')}`,
      );
    }
    return sum + couplingValues[pattern];
  }, 0);

  if (!couplingPatterns[sum]) {
    throw new Error(
      `The joined pattern from ${patterns.join(',')} doesn't exist`,
    );
  }

  return options?.acsFormat ? couplingACSPatterns[sum] : couplingPatterns[sum];
}
