import { toMatchCloseTo } from 'jest-matcher-deep-close-to';

import { NMRSignal1D } from '../../signals/NMRSignal1D';
import { NMRPeak1D } from '../NMRPeak1D';
import { solventSuppression } from '../solventSuppression';

expect.extend({ toMatchCloseTo });

describe('solvent supression', () => {
  const peaks: NMRPeak1D[] = [
    {
      x: 1.01,
      y: 1,
      width: 0.01,
    },
    {
      x: 1.4,
      y: 1,
      width: 0.01,
    },
    {
      x: 1.5,
      y: 2,
      width: 0.01,
    },
    {
      x: 2.01,
      y: 1,
      width: 0.01,
    },
    {
      x: 10,
      y: 2,
      width: 0.01,
    },
    {
      x: 0.5,
      y: 21,
      width: 0.01,
    },
  ];
  it('artificial solvent', () => {
    const solventSignals: NMRSignal1D[] = [
      {
        delta: 1.49,
        js: [{ coupling: 200, multiplicity: 't' }],
        peaks: [
          {
            x: 0.99,
            y: 1.02,
            width: 0.05,
          },
          {
            x: 1.49,
            y: 1.88,
            width: 0.05,
          },
          {
            x: 1.99,
            y: 1.02,
            width: 0.05,
          },
        ],
      },
    ];
    const newPeaks = solventSuppression(peaks, solventSignals);

    expect(newPeaks).toMatchCloseTo([
      { x: 0.5, y: 21, width: 0.01 },
      { x: 1.4, y: 1, width: 0.01 },
      { x: 10, y: 2, width: 0.01 },
    ]);

    const newPeaksMarked = solventSuppression(peaks, solventSignals, {
      markSolventPeaks: true,
    });

    expect(newPeaksMarked).toMatchCloseTo([
      {
        x: 0.5,
        y: 21,
        width: 0.01,
      },
      {
        x: 1.01,
        y: 1,
        width: 0.01,
        kind: 'solvent',
      },
      {
        x: 1.4,
        y: 1,
        width: 0.01,
      },
      {
        x: 1.5,
        y: 2,
        width: 0.01,
        kind: 'solvent',
      },
      {
        x: 2.01,
        y: 1,
        width: 0.01,
        kind: 'solvent',
      },
      {
        x: 10,
        y: 2,
        width: 0.01,
      },
    ]);

    delete solventSignals[0].peaks;
    const newPeaks2 = solventSuppression(peaks, solventSignals);

    expect(newPeaks2).toMatchCloseTo([
      { x: 0.5, y: 21, width: 0.01 },
      { x: 1.4, y: 1, width: 0.01 },
      { x: 10, y: 2, width: 0.01 },
    ]);
  });

  it('doublet solvent signal', () => {
    const solventSignals: NMRSignal1D[] = [
      {
        delta: 1.49,
        js: [{ coupling: 200, multiplicity: 't' }],
        peaks: [
          {
            x: 0.99,
            y: 1.02,
            width: 0.05,
          },
          {
            x: 1.99,
            y: 1.02,
            width: 0.05,
          },
        ],
      },
    ];
    const newPeaks = solventSuppression(peaks, solventSignals);

    expect(newPeaks).toMatchCloseTo([
      { x: 0.5, y: 21, width: 0.01 },
      { x: 1.4, y: 1, width: 0.01 },
      {
        x: 1.5,
        y: 2,
        width: 0.01,
      },
      { x: 10, y: 2, width: 0.01 },
    ]);
  });
});
