import { Shape1D } from 'ml-peak-shape-generator';

import { MakeMandatory } from '../../../utilities/MakeMandatory';
import { convertWidth, NMRPeak1DWithShape } from '../convertWidth';

type NMRPeak1DWithFWHM = NMRPeak1DWithShape & {
  shape: MakeMandatory<Shape1D, 'fwhm'>;
};

describe('convertWidth', () => {
  it('check immutability', () => {
    const peak = {
      x: 2,
      y: 1,
      width: 3,
      shape: {
        kind: 'gaussian',
        fwhm: 3,
      },
    } as NMRPeak1DWithFWHM;

    const newPeaks = convertWidth([peak], { frequency: 2, convertTo: 'hz' });
    const newPeak = newPeaks[0];
    expect(newPeaks[0].width).toBeCloseTo(peak.width * 2);
    expect(newPeak.shape.fwhm).toBeCloseTo(peak.shape.fwhm * 2);
  });
  it('check output options', () => {
    const peak = {
      x: 2,
      y: 1,
      width: 3,
      shape: {
        kind: 'gaussian',
        fwhm: 3,
      },
    } as NMRPeak1DWithFWHM;

    const newPeaks = convertWidth([peak], {
      frequency: 2,
      convertTo: 'hz',
      output: [peak],
    });
    const newPeak = newPeaks[0];
    expect(newPeaks[0].width).toBeCloseTo(peak.width);
    expect(newPeak.shape.fwhm).toBeCloseTo(peak.shape.fwhm);
  });
});
