import { Shape1D } from 'ml-peak-shape-generator';

import { MakeMandatory } from '../../utilities/MakeMandatory';
import { hasProperty } from '../../utilities/hasProperty';
import { NMRPeak1D } from '../NMRPeak1D';

const { parse, stringify } = JSON;

export type NMRPeak1DWithShape = MakeMandatory<NMRPeak1D, 'shape'>;

export interface ConvertWidthOptions<T> {
  convertTo: string & { _kind?: 'hz' | 'ppm' };
  frequency: number;
  output?: T[];
}
/**
 * convert width and fwhm to ppm
 */
export function convertWidth<T extends NMRPeak1D>(
  peaks: T[],
  options: ConvertWidthOptions<T>,
): T[] {
  const {
    frequency,
    convertTo,
    output = parse(stringify(peaks)) as T[],
  } = options;

  const convert = getConverter(convertTo, frequency);

  for (const peak of output) {
    peak.width = convert(peak.width);
    if (hasProperty(peak, 'shape')) {
      const shape = peak.shape as Shape1D;
      if (shape.fwhm) {
        shape.fwhm = convert(shape.fwhm);
      }
    }
  }
  return output;
}

function getConverter(convertTo: string, frequency: number) {
  switch (convertTo) {
    case 'ppm':
      return (x: number) => x / frequency;
    case 'hz':
      return (x: number) => x * frequency;
    default:
      throw new Error(`Does not support convert to ${convertTo}`);
  }
}
