import { FifoLogger } from 'fifo-logger';
import OCL from 'openchemlib/minimal';

import { predictAllSpectra } from '../predictAllSpectra';

import carbonDB from './data/carbonDB-withoutStatistic.json';

jest.setTimeout(50000);

describe('predictAllSpectra', () => {
  test('empty smiles', async () => {
    const smiles = '';
    const logger = new FifoLogger();
    const molecule = OCL.Molecule.fromSmiles(smiles);
    const predictions = await predictAllSpectra(molecule, {
      prediction: {
        logger,
        predictOptions: {
          C: {
            database: carbonDB,
          },
        },
      },
    });
    expect(logger.getLogs()).toHaveLength(2);
    expect(predictions.spectra).toHaveLength(0);
  });

  test('ethylbenzene', async () => {
    const smiles = 'c1ccccc1CC';
    const logger = new FifoLogger();
    const molecule = OCL.Molecule.fromSmiles(smiles);

    const predictions = await predictAllSpectra(molecule, {
      prediction: {
        predictOptions: {
          C: {
            database: carbonDB,
          },
        },
      },
    });
    expect(logger.getLogs()).toHaveLength(0);
    expect(predictions.spectra).toHaveLength(5);
  });
});
