import { fetch } from 'cross-fetch';
import type { Molecule } from 'openchemlib/full';

import type { Prediction1D } from '../Prediction1D';

interface FetchPredictionOptions {
  webserviceURL: string;
}

export async function fetchPrediction(
  molecule: Molecule,
  options: FetchPredictionOptions,
): Promise<Prediction1D> {
  const { webserviceURL } = options;
  const response = await fetch(webserviceURL, {
    headers: {
      accept: 'application/json',
      'content-type': 'application/json',
    },
    body: JSON.stringify({ molfile: molecule.toMolfile() }),
    method: 'POST',
  });

  if (response.status >= 400) {
    throw new Error(response.statusText);
  }
  const prediction = (await response.json()).data;

  prediction.molecule = molecule
    .getOCL()
    .Molecule.fromMolfile(prediction.molfile);
  return prediction;
}
