import type { Molecule } from 'openchemlib/full';
import {
  getHoseCodesFromDiastereotopicID,
  ensureHeterotopicChiralBonds,
  groupDiastereotopicAtomIDs,
} from 'openchemlib-utils';
import type { GroupDiastereotopicAtomIDs } from 'openchemlib-utils';

export interface GroupDiaIDsWithHose extends GroupDiastereotopicAtomIDs {
  hose: string[];
}

export type GroupedDiaIDsWithHose = GroupDiaIDsWithHose[];

export function getFilteredIDiaIDs(
  molecule: Molecule,
  options: { maxSphereSize: number; diaIDs: string[] },
) {
  const { maxSphereSize, diaIDs } = options;

  molecule.addImplicitHydrogens();
  molecule.addMissingChirality();
  ensureHeterotopicChiralBonds(molecule);

  const molfile = molecule.toMolfile();

  const groupedDiaIDs = groupDiastereotopicAtomIDs(diaIDs, molecule, {
    atomLabel: 'C',
  });

  const carbonDiaIDs = groupedDiaIDs.sort((a, b) => {
    if (a.atomLabel === b.atomLabel) {
      return b.counter - a.counter;
    }
    return a.atomLabel < b.atomLabel ? 1 : -1;
  }) as any;

  const OCL = molecule.getOCL();
  for (const diaId of carbonDiaIDs) {
    diaId.hose = getHoseCodesFromDiastereotopicID(
      OCL.Molecule.fromIDCode(diaId.oclID),
      {
        maxSphereSize,
      },
    );
  }

  const toReturn = {
    molfile,
    carbonDiaIDs: carbonDiaIDs as GroupedDiaIDsWithHose,
  };

  return toReturn;
}
