import { couplingPatterns } from '../constants/couplingPatterns';

import type { Jcoupling } from './Jcoupling';
import { NMRSignal1D } from './NMRSignal1D';

export function addDummySignals(signals: NMRSignal1D[]) {
  const newSignals: NMRSignal1D[] = JSON.parse(JSON.stringify(signals));

  let nextAtomToAssign =
    signals.reduce((nextAtomToAssign, signal) => {
      const { atoms = [] } = signal;
      return Math.max(...atoms) > nextAtomToAssign
        ? Math.max(...atoms)
        : nextAtomToAssign;
    }, 0) + 1;
  signals.forEach((signal, s) => {
    const {
      js: jCouplings = [],
      atoms: signalAssignment = [nextAtomToAssign++],
    } = signal;
    const { newCouplings, tempSignals } = checkCouplings(
      jCouplings,
      signalAssignment,
      nextAtomToAssign + signals.length,
    );

    if (tempSignals.length > 0) newSignals.push(...tempSignals);

    newSignals[s].js = newCouplings;
    newSignals[s].atoms = signalAssignment;
  });

  return newSignals.sort((a, b) => a.delta - b.delta);
}

function checkCouplings(
  jCouplings: Jcoupling[],
  signalAssignment: number[],
  nextAtomToAssign: number,
) {
  const tempSignals: NMRSignal1D[] = [];
  const newCouplings = jCouplings.reduce<Jcoupling[]>(
    (newCouplings, jCoupling) => {
      const { atoms = [], multiplicity, coupling } = jCoupling;
      if (atoms.length === 0) {
        if (coupling && multiplicity) {
          const tempCouplings: Jcoupling[] = [];
          const nbLinks = couplingPatterns.indexOf(multiplicity);
          for (let i = 0; i < nbLinks; i++) {
            nextAtomToAssign++;
            tempCouplings.push({
              coupling,
              atoms: [nextAtomToAssign],
            });
            tempSignals.push(
              formatSignal(coupling, [nextAtomToAssign], signalAssignment),
            );
          }
          newCouplings.push(...tempCouplings);
        } else {
          newCouplings.push(jCoupling);
        }
      }
      return newCouplings;
    },
    [],
  );
  return { newCouplings, tempSignals };
}

function formatSignal(
  coupling: number,
  newSignalAssignment: number[],
  signalAssignment: number[],
) {
  return {
    delta: 100000,
    atoms: newSignalAssignment,
    js: [
      {
        coupling,
        atoms: signalAssignment,
      },
    ],
  };
}
