import { toBeDeepCloseTo } from 'jest-matcher-deep-close-to';
import { MakeMandatory } from 'ml-gsd/lib/utils/MakeMandatory';

import { Signal } from '../../optimizeSignals';
import { defaultParameters } from '../defaultParameters';
import { Shape1DWithFWHM } from '../getInternalSignals';

expect.extend({ toBeDeepCloseTo });
describe('default parameter in optimizeSignals', () => {
  const signal: MakeMandatory<Signal, 'js'> = {
    delta: 1,
    js: [{ multiplicity: 't', coupling: 5 }],
  };
  const shape: Shape1DWithFWHM = { kind: 'gaussian', fwhm: 2 };
  const properties = ['init', 'min', 'max', 'gradientDifference'];
  const parameters = ['delta', 'intensity', 'fwhm', 'coupling'];
  const expectedResult: Record<string, Record<string, number>> = {
    delta: {
      init: signal.delta,
      min: signal.delta - shape.fwhm * 2,
      max: signal.delta + shape.fwhm * 2,
      gradientDifference: 2e-8,
    },
    intensity: {
      init: 1,
      min: 0,
      max: 1.5,
      gradientDifference: 1e-3,
    },
    fwhm: {
      init: shape.fwhm,
      min: shape.fwhm * 0.25,
      max: shape.fwhm * 4,
      gradientDifference: shape.fwhm * 2e-3,
    },
    coupling: {
      init: signal.js[0].coupling,
      min: signal.js[0].coupling * 0.8,
      max: signal.js[0].coupling * 1.2,
      gradientDifference: 1e-2,
    },
  };
  const arrayTest = [];
  for (const parameter of parameters) {
    const input =
      parameter === 'coupling'
        ? { jCoupling: signal.js[0] }
        : { signal, shape };

    for (const property of properties) {
      const defaultParameterFunct = defaultParameters[parameter][property];
      arrayTest.push({
        result: defaultParameterFunct(input),
        expected: expectedResult[parameter][property],
        parameter,
        property,
      });
    }
  }
  it.each(arrayTest)(
    'compare values of property $property of the parameter $parameter',
    ({ result, expected }) => {
      expect(result).toBeDeepCloseTo(expected, 2);
    },
  );
});
