import { toMatchCloseTo } from 'jest-matcher-deep-close-to';

import { hackSignalsToXY } from '../../hackSignalsToXY';
import { Signal } from '../../optimizeSignals';
import { getInternalSignals } from '../getInternalSignals';

expect.extend({ toMatchCloseTo });

describe('getInternalSignals', () => {
  it('one signal', () => {
    const frequency = 100;
    const nbPoints = 1024;
    const delta = 0.5;
    const coupling = 8;
    const lineWidth = 2;
    const maxValue = 2;
    const multiplicity = 'd';
    const minMaxY = {
      min: 0,
      max: maxValue,
      range: maxValue,
    };
    const [from, to] = [delta - 0.1, delta + 0.1];
    const signal: Signal = {
      delta,
      js: [{ multiplicity, coupling }],
    };
    const dataXY = hackSignalsToXY([signal], {
      frequency,
      nbPoints,
      from,
      to,
      lineWidth,
      maxValue,
    });
    const internalSignal = getInternalSignals(dataXY, [signal], minMaxY, {
      frequency,
      lineWidth,
    })[0];
    expect(internalSignal.signal.intensity).toBeCloseTo(1);
    expect(internalSignal.shape).toMatchCloseTo({
      kind: 'gaussian',
      fwhm: lineWidth / frequency,
    });
    expect(internalSignal.fromIndex).toBe(0);
    expect(internalSignal.toIndex).toBe(2);
    expect(internalSignal.toIndexCoupling).toBe(3);
    expect(internalSignal.parameters).toHaveLength(3);
    const { propertiesValues } = internalSignal;
    expect(propertiesValues.init[0]).toBe(delta);
    expect(propertiesValues.init[1]).toBe(1);
  });
  it('one signal with intensity and shape', () => {
    const frequency = 100;
    const nbPoints = 1024;
    const delta = 0.5;
    const coupling = 8;
    const lineWidth = 2;
    const maxValue = 2;
    const multiplicity = 'd';
    const minMaxY = {
      min: 0,
      max: maxValue,
      range: maxValue,
    };
    const [from, to] = [delta - 0.1, delta + 0.1];
    const signal: Signal = {
      delta,
      shape: { kind: 'lorentzian' },
      intensity: 2,
      js: [{ multiplicity, coupling }],
    };
    const dataXY = hackSignalsToXY([signal], {
      frequency,
      nbPoints,
      from,
      to,
      lineWidth,
      maxValue,
    });
    const internalSignal = getInternalSignals(dataXY, [signal], minMaxY, {
      frequency,
      lineWidth,
    })[0];
    expect(internalSignal.signal.intensity).toBeCloseTo(1);
    expect(internalSignal.shape).toMatchCloseTo({
      kind: 'lorentzian',
      fwhm: lineWidth / frequency,
    });
    expect(internalSignal.fromIndex).toBe(0);
    expect(internalSignal.toIndex).toBe(2);
    expect(internalSignal.toIndexCoupling).toBe(3);
    expect(internalSignal.parameters).toHaveLength(3);
    expect(internalSignal.signal.intensity).toBe(1);
    const { propertiesValues } = internalSignal;
    expect(propertiesValues.init[0]).toBe(delta);
    expect(propertiesValues.init[1]).toBe(1);
  });
});
