import { PseudoVoigtShape1D, Shape1D } from 'ml-peak-shape-generator';

import { Jcoupling } from '../Jcoupling';
import { Signal } from '../optimizeSignals';

type Shape1DWithFWHM = Omit<Shape1D, 'fwhm'> & { fwhm: number };

export const defaultParameters: Record<
  string,
  Record<string, (...args: any) => number>
> = {
  delta: {
    init: (data: { signal: Signal }) => data.signal.delta,
    min: (data: { signal: Signal; shape: Shape1DWithFWHM }) =>
      data.signal.delta - data.shape.fwhm * 2,
    max: (data: { signal: Signal; shape: Shape1DWithFWHM }) =>
      data.signal.delta + data.shape.fwhm * 2,
    gradientDifference: () => 2e-3,
  },
  intensity: {
    init: (data: { signal: Signal }) => data.signal.intensity || 1,
    min: () => 0,
    max: () => 1.5,
    gradientDifference: () => 1e-3,
  },
  fwhm: {
    init: (data: { shape: Shape1DWithFWHM }) => data.shape.fwhm,
    min: (data: { shape: Shape1DWithFWHM }) => data.shape.fwhm * 0.25,
    max: (data: { shape: Shape1DWithFWHM }) => data.shape.fwhm * 4,
    gradientDifference: (data: { shape: Shape1DWithFWHM }) =>
      data.shape.fwhm * 2e-3,
  },
  mu: {
    init: (data: { shape: PseudoVoigtShape1D }) => data.shape.mu || 0.5,
    min: () => 0,
    max: () => 1,
    gradientDifference: () => 0.01,
  },
  coupling: {
    init: (data: { jCoupling: Jcoupling }) => data.jCoupling.coupling,
    min: (data: { jCoupling: Jcoupling }) => data.jCoupling.coupling * 0.8,
    max: (data: { jCoupling: Jcoupling }) => data.jCoupling.coupling * 1.2,
    gradientDifference: () => 0.01,
  },
};
