import { DataXY } from 'cheminfo-types';
import direct from 'ml-direct';

import { SumOfShapes } from './getSumOfShapes';

interface DirectOptions {
  iterations?: number;
  epsilon?: number;
  tolerance?: number;
  tolerance2?: number;
  initialState?: any;
}

interface DirectOptimizationOptions extends DirectOptions {
  directOptions?: DirectOptions;
  minValues: number[];
  maxValues: number[];
}

export function directOptimization(
  data: DataXY,
  sumOfShapes: SumOfShapes,
  options: DirectOptimizationOptions,
) {
  const { minValues, maxValues, directOptions = {} } = options;
  const objectiveFunction = getObjectiveFunction(data, sumOfShapes);
  const result = direct(objectiveFunction, minValues, maxValues, directOptions);

  const { optima } = result;

  return {
    error: result.minFunctionValue,
    iterations: result.iterations,
    parameterValues: optima[0],
  };
}

function getObjectiveFunction(data: DataXY, sumOfShapes: SumOfShapes) {
  const { x, y } = data;
  const nbPoints = x.length;
  return (parameters: number[]) => {
    const fct = sumOfShapes(parameters);
    let error = 0;
    for (let i = 0; i < nbPoints; i++) {
      error += Math.pow(y[i] - fct(x[i]), 2);
    }
    return error;
  };
}
