import { gsd } from 'ml-gsd';
import Matrix from 'ml-matrix';

import { NMRRange } from '../../..';
import { peaksToRanges } from '../../../peaks/peaksToRanges';
import { simulate1D } from '../simulate1D';

test('2 singulets', () => {
  const result = simulate1D(
    {
      clusters: [[0, 1]],
      couplingConstants: new Matrix([
        [0, 0],
        [0, 0],
      ]),
      chemicalShifts: [1, 2],
      levels: [2, 2],
      connectivity: new Matrix([
        [0, 1],
        [1, 0],
      ]),
    },
    {
      lineWidth: 1,
      frequency: 400,
      from: 0,
      to: 10,
      nbPoints: 1024,
      shape: {
        kind: 'gaussian',
      },
    },
  );
  const peaks = gsd(result);
  expect(peaks).toHaveLength(2);
  const ranges = peaksToRanges(result, peaks);
  expect(ranges).toHaveLength(2);
});

test('2 singulets without clusters', () => {
  const result = simulate1D(
    {
      couplingConstants: new Matrix([
        [0, 0],
        [0, 0],
      ]),
      chemicalShifts: [1, 2],
      levels: [2, 2],
    },
    {
      lineWidth: 1,
      frequency: 400,
      from: 0,
      to: 10,
      nbPoints: 1024,
      shape: {
        kind: 'gaussian',
      },
    },
  );
  const peaks = gsd(result);
  expect(peaks).toHaveLength(2);
  const ranges = peaksToRanges(result, peaks);
  expect(ranges).toHaveLength(2);
});

test('2 doublets and 1 triplet', () => {
  const result = simulate1D(
    {
      clusters: [[0, 1, 2]],
      couplingConstants: new Matrix([
        [0, 10, 0],
        [10, 0, 10],
        [0, 10, 0],
      ]),
      chemicalShifts: [1, 2, 3],
      levels: [2, 2, 2],
      connectivity: new Matrix([
        [1, 1, 0],
        [1, 1, 1],
        [0, 1, 1],
      ]),
    },
    {
      from: 0.5,
      to: 3.5,
      nbPoints: 1024 * 2,
    },
  );
  const peaks = gsd(result);
  expect(peaks).toHaveLength(7);
  const ranges = peaksToRanges(result, peaks) as Array<Required<NMRRange>>;
  expect(ranges).toHaveLength(3);
  expect(ranges[0].signals[0].multiplicity).toStrictEqual('d');
  expect(ranges[1].signals[0].multiplicity).toStrictEqual('t');
  expect(ranges[2].signals[0]?.multiplicity).toStrictEqual('d');
});

test('2 doublets and 1 triplet, number[][]', () => {
  const result = simulate1D(
    {
      clusters: [[0, 1, 2]],
      couplingConstants: [
        [0, 10, 0],
        [10, 0, 10],
        [0, 10, 0],
      ],
      chemicalShifts: [1, 2, 3],
      levels: [2, 2, 2],
      connectivity: [
        [1, 1, 0],
        [1, 1, 1],
        [0, 1, 1],
      ],
    },
    {
      from: 0.5,
      to: 3.5,
      nbPoints: 1024 * 2,
    },
  );
  const peaks = gsd(result);
  expect(peaks).toHaveLength(7);
  const ranges = peaksToRanges(result, peaks) as Array<Required<NMRRange>>;
  expect(ranges).toHaveLength(3);
  expect(ranges[0].signals[0].multiplicity).toStrictEqual('d');
  expect(ranges[1].signals[0].multiplicity).toStrictEqual('t');
  expect(ranges[2].signals[0]?.multiplicity).toStrictEqual('d');
});

test('2 doublets and 1 triplet', () => {
  const result = simulate1D(
    {
      couplingConstants: new Matrix([
        [0, 10, 0],
        [10, 0, 10],
        [0, 10, 0],
      ]),
      chemicalShifts: [1, 2, 3],
      levels: [2, 2, 2],
    },
    {
      from: 0.5,
      to: 3.5,
      nbPoints: 1024 * 2,
    },
  );
  const peaks = gsd(result);
  expect(peaks).toHaveLength(7);
  const ranges = peaksToRanges(result, peaks) as Array<Required<NMRRange>>;
  expect(ranges).toHaveLength(3);
  expect(ranges[0].signals[0].multiplicity).toStrictEqual('d');
  expect(ranges[1].signals[0].multiplicity).toStrictEqual('t');
  expect(ranges[2].signals[0]?.multiplicity).toStrictEqual('d');
});

test('2 doublets and 1 triplet, number[][]', () => {
  const result = simulate1D(
    {
      couplingConstants: [
        [0, 10, 0],
        [10, 0, 10],
        [0, 10, 0],
      ],
      chemicalShifts: [1, 2, 3],
      levels: [2, 2, 2],
    },
    {
      from: 0.5,
      to: 3.5,
      nbPoints: 1024 * 2,
    },
  );
  const peaks = gsd(result);
  expect(peaks).toHaveLength(7);
  const ranges = peaksToRanges(result, peaks) as Array<Required<NMRRange>>;
  expect(ranges).toHaveLength(3);
  expect(ranges[0].signals[0].multiplicity).toStrictEqual('d');
  expect(ranges[1].signals[0].multiplicity).toStrictEqual('t');
  expect(ranges[2].signals[0]?.multiplicity).toStrictEqual('d');
});
