import { toBeDeepCloseTo, toMatchCloseTo } from 'jest-matcher-deep-close-to';

import { resurrectRange } from '../resurrectRange';

expect.extend({ toBeDeepCloseTo, toMatchCloseTo });

describe('resurrectRange', () => {
  it('1 (t, 3H, J=7.0Hz)', () => {
    const range = resurrectRange('1 (t, 3H, J=7.0Hz)');
    expect(range).toMatchInlineSnapshot(`
      {
        "from": 0.97875,
        "integration": 3,
        "signals": [
          {
            "delta": 1,
            "js": [
              {
                "coupling": 7,
                "multiplicity": "t",
              },
            ],
            "multiplicity": "t",
          },
        ],
        "to": 1.02125,
      }
    `);
  });
  it('1.5 (hept,  2H, J=7.0Hz)', () => {
    const range = resurrectRange('1.5 (hept,  2H, J=7.0Hz)');
    expect(range).toMatchInlineSnapshot(`
      {
        "from": 1.44375,
        "integration": 2,
        "signals": [
          {
            "delta": 1.5,
            "js": [
              {
                "coupling": 7,
                "multiplicity": "hept",
              },
            ],
            "multiplicity": "hept",
          },
        ],
        "to": 1.55625,
      }
    `);
  });
  it('1.5-1.6 (m, 2H)', () => {
    const range = resurrectRange('1.5-1.6 (m, 2H)');
    expect(range).toStrictEqual({
      from: 1.5,
      to: 1.6,
      signals: [{ delta: 1.55, js: [], multiplicity: 'm' }],
      integration: 2,
    });
  });
  it('-1.6--1.5 (m, 2H)', () => {
    const range = resurrectRange('-1.6--1.5 (m, 2H)');
    expect(range).toStrictEqual({
      from: -1.6,
      to: -1.5,
      signals: [{ delta: -1.55, js: [], multiplicity: 'm' }],
      integration: 2,
    });
  });
  it('∂: -1.6--1.5 (m, 2H)', () => {
    const range = resurrectRange('∂: -1.6--1.5 (m, 2H)');
    expect(range).toStrictEqual({
      from: -1.6,
      to: -1.5,
      signals: [{ delta: -1.55, js: [], multiplicity: 'm' }],
      integration: 2,
    });
  });
  it('-1.6 (m, 2H)', () => {
    const range = resurrectRange('-1.6 (m, 2H)');
    expect(range).toMatchCloseTo({
      from: -1.6 - 1 / 40,
      to: -1.6 + 1 / 40,
      signals: [{ delta: -1.6, js: [], multiplicity: 'm' }],
      integration: 2,
    });
  });
});
