import { gyromagneticRatio, Nuclei } from 'gyromagnetic-ratio';

export interface ObjervedNucleusData {
  /**
   * nucleus to use as reference in the calculation of frequency
   */
  nucleus: Nucleus;
  /**
   * frequency of the reference nucleus
   */
  frequency: number;
}

//Literal type is extends string and when union it with string we lose the autocomplete and to solve this we distinguish string
type Nucleus = Nuclei | (string & Record<never, never>);

/**
 * calculate the frequency of a nucleus with respect to a reference nucleus
 */
export function getFrequency(
  /**
   * nucleus to calculate the frequency
   */
  nucleus: Nucleus,
  observedNucleusData: ObjervedNucleusData,
) {
  const { nucleus: obsNucleus, frequency } = observedNucleusData;

  checkExistence(nucleus, obsNucleus);

  return (
    (frequency * gyromagneticRatio[nucleus as Nuclei]) /
    gyromagneticRatio[obsNucleus as Nuclei]
  );
}

function checkExistence(...nucleus: Nucleus[]) {
  const nuclei: string[] = [];
  for (const n of nucleus) {
    if (!(n in gyromagneticRatio)) {
      nuclei.push(n);
    }
  }
  if (nuclei.length !== 0) {
    throw new Error(`${nuclei.join(' , ')} is not supported`);
  }
}
