import { Nuclei, gyromagneticRatio } from 'gyromagnetic-ratio';

import { NucleusName, nucleusSymbolMap } from './nucleusMap';

/**
 * The idea is to convert nucleus name like H1, proton or Fluorine19 to 1H and 19F.
 */

export function normalizeNucleus(name: string) {
  if (gyromagneticRatio[name as Nuclei]) {
    return name;
  }

  const massNumber = name.replace(/\D/g, '');
  const atomName = name.replace(/\d*\W*/g, '').toLowerCase();

  const nucleusName =
    atomName.length > 1
      ? nucleusSymbolMap[atomName as NucleusName]
      : atomName.toUpperCase();

  const symbol = `${massNumber}${nucleusName}`;
  if (!gyromagneticRatio[symbol as Nuclei]) {
    return name;
  }

  return symbol;
}
