import { couplingValues } from '../constants/couplingValues';
import type { NMRSignal1D } from '../signals/NMRSignal1D';

export interface RangeFromSignalOptions {
  /**
   * nucleus
   * @default '1h'
   */
  nucleus?: string;
  /**
   * frequency observedhttps://www.wiley.com/en-us/NMR+Data+Processing-p-9780471039006#content-section
   * @default 400
   */
  frequency?: number;
  /**
   * tolerance to the halfWidth, default depends of the nucleus
   */
  tolerance?: number;
}

export function rangeFromSignal(
  signal: NMRSignal1D,
  options: RangeFromSignalOptions = {},
) {
  const { nucleus = '1h', frequency = 400 } = options;
  const { tolerance = getTolerance(nucleus) / frequency } = options;

  let halfWidth = 0;
  const { js: jCouplings = [], multiplicity } = signal;
  if (jCouplings.length > 0) {
    for (const js of jCouplings) {
      const { coupling, multiplicity = 'd' } = js;
      halfWidth += (couplingValues[multiplicity] * coupling) / frequency;
    }
  } else if (multiplicity === 'm') {
    halfWidth = 16 / frequency;
  } else if (multiplicity === 'b' || multiplicity === 'br s') {
    halfWidth = 7 / frequency;
  }

  halfWidth = tolerance + halfWidth / 2;

  return {
    from: signal.delta - halfWidth,
    to: signal.delta + halfWidth,
  };
}

function getTolerance(nucleus: string) {
  switch (nucleus.toLocaleLowerCase()) {
    case '1h':
      return 1.5;
    case '13C':
      return 3;
    default:
      return 2;
  }
}
