import { DataXY, PeakXYWidth } from 'cheminfo-types';
import { optimizePeaks, OptimizePeaksOptions } from 'ml-gsd';

import { convertWidthToHz } from '../peaks/util/convertWidthToHz';
import { convertWidthToPPM } from '../peaks/util/convertWidthToPPM';

export interface XYPeaksOptimizationOptions extends OptimizePeaksOptions {
  /**
   * frequency to convert width and fwhm to ppm.
   */
  frequency: number;
}

export function xyPeaksOptimization<T extends PeakXYWidth>(
  data: DataXY,
  peaks: T[],
  options: XYPeaksOptimizationOptions,
) {
  const { frequency } = options;
  const newPeaks = convertWidthToPPM(peaks, { frequency });
  const optimizedPeaks = optimizePeaks(data, newPeaks, options);
  const result = convertWidthToHz(optimizedPeaks, {
    frequency,
    output: optimizedPeaks,
  });
  return result;
}
