import { NmrData2D, DoubleArray } from 'cheminfo-types';
import { xMultiply } from 'ml-spectra-processing';

import { checkData2DFid, Data2DFidComplete } from '../utilities/checkData2DFid';

export function quadrature(data: NmrData2D, options: { fnMode: string }) {
  checkData2DFid(data);

  const { fnMode } = options;

  switch (fnMode.toLowerCase()) {
    case 'ttpi':
      return ttpiQuadrature(data);
    case 'states':
      return shrQuadrature(data);
    default:
      return {
        re: data.re.z,
        im: data.im.z,
      };
  }
}

function shrQuadrature(data: Data2DFidComplete) {
  const { re, im } = data;
  return { re: re.z, im: im.z };
}

function ttpiQuadrature(data: Data2DFidComplete) {
  const newRe: DoubleArray[] = [];
  const newIm: DoubleArray[] = [];

  const {
    re: { z: reMatrix },
    im: { z: imMatrix },
  } = data;
  for (let i = 0; i < reMatrix.length; i++) {
    newRe.push(i % 2 ? reMatrix[i].slice() : xMultiply(reMatrix[i], -1));
    newIm.push(i % 2 ? imMatrix[i].slice() : xMultiply(imMatrix[i], -1));
  }

  return { re: newRe, im: newIm };
}
