import { NmrData2DFt } from 'cheminfo-types';
import Matrix from 'ml-matrix';

import { PhaseCorrectionOptions } from '../../xyzPhaseCorrectionDirectDimension';

import { checkNmrData2DFourQuad } from './checkNmrData2DFourQuad';
import { getZMinMax } from './getZMinMax';

export function applyOverImag<T extends NmrData2DFt>(
  data: T,
  options: PhaseCorrectionOptions,
) {
  checkNmrData2DFourQuad(data);
  const { applyPhaseCorrection } = options;
  const { re, im } = applyPhaseCorrection(
    { re: Matrix.checkMatrix(data.ri.z), im: Matrix.checkMatrix(data.ii.z) },
    options,
  );

  return {
    ...data,
    ri: {
      ...data.ri,
      ...getZMinMax(re),
    },
    ii: {
      ...data.ii,
      ...getZMinMax(im),
    },
  };
}
