import { NmrData2DFt } from 'cheminfo-types';
import Matrix from 'ml-matrix';

import { PhaseCorrectionOptions } from '../../xyzPhaseCorrectionDirectDimension';

import { checkNmrData2DTwoQuad } from './checkNmrData2DTwoQuad';
import { getZMinMax } from './getZMinMax';

export function applyOverReal<T extends NmrData2DFt>(
  data: T,
  options: PhaseCorrectionOptions,
) {
  const { applyPhaseCorrection } = options;
  checkNmrData2DTwoQuad(data);
  const { re, im } = applyPhaseCorrection(
    { re: Matrix.checkMatrix(data.rr.z), im: Matrix.checkMatrix(data.ir.z) },
    options,
  );

  return {
    ...data,
    rr: {
      ...data.rr,
      ...getZMinMax(re),
    },
    ir: {
      ...data.ir,
      ...getZMinMax(im),
    },
  };
}
