import { NmrData2DContent } from 'cheminfo-types';

import { xyzAutoSignalsPicking } from '..';
import type { GetKernelOptions } from '../peaks/util/getKernel';
import { MakeMandatory } from '../utilities/MakeMandatory';

import { NMRZone } from './NMRZone';
import { formatZones } from './util/formatZone';

export interface XYZAutoZonesPickingOptions {
  /**
   * max number of points in any dimension to pad the input data, this is needed to avoid lost peaks when the input matrix is too small.
   * @default 14
   */
  sizeToPad?: number;
  /**
   * If it's true, try to find the real maximum for each bidimensional peak.
   * @default true
   */
  realTopDetection?: boolean;
  /**
   * positive value filter small peaks by intensity
   * @default 0.5
   */
  thresholdFactor?: number;
  /**
   * specify the nuclei of each dimension
   * @default ['1H','1H']
   */
  nuclei?: string[];
  /**
   * specify the frequency for each dimension.
   */
  observedFrequencies: number[] | Float64Array;
  /**
   * If it's true, it try to ajust the shift and add missing cross peaks, useful only for homonuclear data.
   * @default false
   */
  enhanceSymmetry?: boolean;
  /**
   * If it's true, peaks with less than a specific percentage (maxPercentCutOff) of the maximum intensity will be removed.
   * @default true;
   */
  clean?: boolean;
  /**
   * percentage of maximum intensity as cutoff for small peaks.
   * @default 0.03
   */
  maxPercentCutOff?: number;
  /**
   * Specify the minimum distance in Hz for each dimension to join peaks in a signal.
   * @default [24,24]
   */
  tolerances?: number[];
  /**
   * If it is true, the convolution will be forced by FFT
   * @default true
   */
  convolutionByFFT?: boolean;
  /**
   * kernel options to calculate a laplacianOfGaussian kernel with a specify size.
   */
  kernel?: GetKernelOptions;
}
export type NMRZoneWithID = MakeMandatory<NMRZone, 'id'>;

export function xyzAutoZonesPicking(
  spectraData: NmrData2DContent,
  options: XYZAutoZonesPickingOptions,
): NMRZoneWithID[] {
  const signals = xyzAutoSignalsPicking(spectraData, options);
  return formatZones(signals);
}
