import { NmrData2DFt } from 'cheminfo-types';
import Matrix from 'ml-matrix';
import { reimPhaseCorrection } from 'ml-spectra-processing';

import { degreeToRadians } from '../constants/degreeToRadians';

import { applyOverImag } from './util/phaseCorrection/applyOverImag';
import { applyOverReal } from './util/phaseCorrection/applyOverReal';

interface ReimMatrix {
  re: Matrix;
  im: Matrix;
}
export interface PhaseCorrectionOptions {
  ph0: number;
  ph1: number;
  applyPhaseCorrection: (
    d: ReimMatrix,
    o: { ph0: number; ph1: number },
  ) => ReimMatrix;
}

export function xyzPhaseCorrectionDirectDimension(
  data: NmrData2DFt,
  options: PhaseCorrectionOptions,
) {
  applyOverReal(data, { ...options, applyPhaseCorrection });
  if ('ri' in data && 'ii' in data) {
    applyOverImag(data, { ...options, applyPhaseCorrection });
  }
  return data;
}

function applyPhaseCorrection(
  data: { re: Matrix; im: Matrix },
  options: { ph0: number; ph1: number },
) {
  const ph0 = options.ph0 * degreeToRadians;
  const ph1 = options.ph1 * degreeToRadians;
  const nbRows = data.re.rows;
  for (let i = 0; i < nbRows; i++) {
    const { re, im } = reimPhaseCorrection(
      {
        re: data.re.getRow(i),
        im: data.im.getRow(i),
      },
      ph0,
      ph1,
    );
    data.re.setRow(i, re);
    data.im.setRow(i, im);
  }
  return data;
}
