import { NmrData2DFt } from 'cheminfo-types';
import Matrix from 'ml-matrix';
import { reimPhaseCorrection } from 'ml-spectra-processing';

import { degreeToRadians } from '../constants/degreeToRadians';

import { applyOverImag } from './util/phaseCorrection/applyOverImag';
import { applyOverReal } from './util/phaseCorrection/applyOverReal';
import { PhaseCorrectionOptions } from './xyzPhaseCorrectionDirectDimension';

export type PhaseCorrectionIndirectDimensionOptions = PhaseCorrectionOptions & {
  quad: 'imag' | 'real';
};

export function xyzPhaseCorrectionIndirectDimension(
  data: NmrData2DFt,
  options: PhaseCorrectionIndirectDimensionOptions,
) {
  const { quad } = options;
  if (quad === 'real') {
    return applyOverReal(data, { ...options, applyPhaseCorrection });
  } else if (quad === 'imag') {
    return applyOverImag(data, { ...options, applyPhaseCorrection });
  }
}

function applyPhaseCorrection(
  data: { re: Matrix; im: Matrix },
  options: { ph0: number; ph1: number },
) {
  const ph0 = options.ph0 * degreeToRadians;
  const ph1 = options.ph1 * degreeToRadians;
  const nbColumns = data.re.columns;
  for (let i = 0; i < nbColumns; i++) {
    const { re, im } = reimPhaseCorrection(
      {
        re: data.re.getColumn(i),
        im: data.im.getColumn(i),
      },
      ph0,
      ph1,
    );
    data.re.setColumn(i, re);
    data.im.setColumn(i, im);
  }
  return data;
}
